/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Copyright (C) 2010 France Telecom
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.monitoring.service;

import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

import java.util.Date;

/**
 * Extractor used for Replay
 * @author durieuxp
 */
public abstract class Extractor extends Thread {

    /**
     * Logger.
     */
    protected final static Log logger = LogFactory.getLog(MonitoringService.class);

    protected DashboardDataCollector dbc;

    protected Date from;

    protected Date to;

    protected boolean stopWanted = false;

    /**
     * Constructor
     * @param dbc
     */
    public Extractor(DashboardDataCollector dbc) {
        this.dbc = dbc;
    }

    /**
     * Set date limits for the thread collecting data.
     * Must be set before running the thread.
     * @param from
     * @param to
     */
    public void setDateLimits(Date from, Date to) {
        this.from = from;
        this.to = to;
    }

    public void stopMonitoring() {
        stopWanted = true;
    }

    /**
     * Check if Attr matches the ycol field
     * @param attr Attribute found in the result
     * @param ycol Definition in the y.col field of configuration file.
     * @return true if matches
     */
    protected boolean attrmatch(String attr, String ycol) {
        if (attr.equals(ycol)) {
            // match exactly
            return true;
        }
        int index = attr.indexOf(".");
        if (index > 0 && attr.substring(index + 1).equals(ycol)) {
            // matches the attribute name
            // Configuration did not care about the indicator.
            return true;
        }
        //logger.debug("attr do not match:" + ycol);
        return false;
    }
}
