/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Copyright (C) 2010 France Telecom
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.monitoring.service;

import org.ow2.jasmine.monitoring.eos.monitoring.EventData;
import org.ow2.jasmine.monitoring.eos.monitoring.Serie;

import java.io.BufferedReader;
import java.io.IOException;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * Extractor for File with header
 * @deprecated was used by mbeancmd
 * @author durieuxp
 */
public class HeaderFileExtractor extends Extractor {

    private String header;

    private BufferedReader buffer;

    private int dateind;

    private long fromTimestamp = 0L;

    private long toTimestamp = 0L;

    // ------------------------------------------------------------------------
    // Constructors
    // ------------------------------------------------------------------------

    /**
     * Constructor for DashboardDataCollector
     */
    public HeaderFileExtractor(DashboardDataCollector dbc, String header, int dateind, BufferedReader buffer, Date from, Date to) {
        super(dbc);
        setDateLimits(from, to);
        this.header = header;
        this.buffer = buffer;
        this.dateind = dateind;
        if (from != null) {
            this.fromTimestamp = from.getTime();
        }
        if (to != null) {
            this.toTimestamp = to.getTime();
        }
    }

    // ------------------------------------------------------------------------
    // Thread implementation
    // ------------------------------------------------------------------------

    /**
     * Run method called when the thread is started.
     */
    public void run() {
        logger.debug("start reading header file");

        // Parse the header to look for column names
        String[] headerArray = header.split(";");
        int mbeanCol = -1;
        int serverCol = -1;
        int timestampCol = -1;
        Map<String, Integer> valueCol = new HashMap<String, Integer>();
        for (int index = 0; index < headerArray.length; index++) {
            if (headerArray[index].equals("server")) {
                serverCol = index;
            }
            if (headerArray[index].equals("mbean")) {
                mbeanCol = index;
            }
            if (headerArray[index].equals("time")) {
                timestampCol = index;
            }
            for (Serie afil : dbc.getSeries()) {
                if (afil.getYcol().equals(headerArray[index])) {
                    valueCol.put(afil.getId(), new Integer(index));
                }
            }
        }
        if (serverCol == -1) {
            logger.error("Bad log file: No server in header");
        }
        if (mbeanCol == -1) {
            logger.error("Bad log file: No mbean in header");
        }
        if (timestampCol == -1) {
            logger.error("Bad log file: No time in header");
        }


        // parse the file
        String line = "";
        long timestamp = 0;
        try {
            while (! stopWanted) {
                line = buffer.readLine();
                if (line == null) {
                    if (timestamp >= toTimestamp) {
                        logger.debug("End of file");
                        break;
                    } else {
                        logger.debug("Wait for new event");
                        Thread.sleep(5000);
                        continue;
                    }
                }
                // split the line just once
                String[] split = line.split(";");
                if (split.length < headerArray.length) {
                    // Ignore all lines incomplete.
                    // This may happen in case of "poll -slb" for example.
                    continue;
                }
                String date = split[dateind];
                String server = split[serverCol];
                timestamp = new Long(split[timestampCol]);

                // Check date
                if (timestamp < fromTimestamp) {
                    // logger.debug("too old");
                    continue;
                }
                if (toTimestamp > 0 && timestamp > toTimestamp) {
                    logger.debug("ignoring the rest of the file : not in the time range");
                    break;
                }

                // compute mbean
                String mbean = split[mbeanCol];

                // Find series accepting this event.
                for (Serie curser : dbc.getSeries()) {
                    Integer valueIndex = valueCol.get(curser.getId());
                    if (valueIndex == null) {
                        // This serie is not present in this file.
                        // logger.debug("This serie is not present in this file");
                        continue;
                    }
                    String value = split[valueIndex.intValue()];

                    // Check pattern
                    if (! dbc.match(mbean, curser.getPattern())) {
                        // logger.debug("bad pattern:" + curser.getPattern());
                        continue;
                    }

                    // Check server
                    if (curser.getServer() != null && curser.getServer().length() > 0 && !curser.getServer().equals(server)) {
                        // logger.debug("bad server:" + curser.getServer());
                        continue;
                    }

                    // This event matches this Serie and will be kept
                    EventData ed = new EventData(curser.getId(), timestamp, value);
                    // logger.debug("found a result");
                    dbc.addResult(ed);
                    // continue because an event may match several series
                }
            }
        } catch (IOException e) {
            logger.error("Error reading file: " + e);
        } catch (InterruptedException e) {
            logger.warn("Interrupted");
        } finally {
            dbc.setReplayCompleted();
        }

    }
}
